import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter_query/flutter_query.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

void main() {
  testWidgets('useQuery', (tester) async {
    var passed = false;

    await tester.pumpFrames(
      QueryCacheProvider(
        child: HookBuilder(
          builder: (context) {
            final result = useQuery(
              'data',
              () => Future.value('Hello!'),
              onSuccess: (data) {
                print('Success!');
              },
              onError: (err) {
                print('Shouldnt hit here!');
              },
            );
            if (result.isSuccess && result.data == 'Hello!') {
              passed = true;
            }
            return Container();
          },
        ),
      ),
      Duration(seconds: 5),
    );

    expect(passed, true);
  });

  testWidgets('useInfiniteQuery', (tester) async {
    var passed = false;
    var index = 0;

    await tester.pumpFrames(
      QueryCacheProvider(
        child: HookBuilder(
          builder: (context) {
            final result = useInfiniteQuery<List<String>>(
              'data',
              (page) {
                index = page + 1;
                final value =
                    page > 2 ? List<String>.empty() : [page.toString()];
                return Future<List<String>>.value(value);
              },
              getFetchMore: (latestPage, allPages) {
                if (latestPage is List && latestPage.isEmpty) {
                  return false;
                } else {
                  return index;
                }
              },
            );

            if (!result.canFetchMore && !result.isLoading) {
              passed = true;
            }
            if (result.isSuccess && result.canFetchMore) {
              result.fetchMore();
            }
            return Container();
          },
        ),
      ),
      Duration(seconds: 5),
    );

    expect(passed, true);
  });

  testWidgets('useInfiniteQuery invalidate', (tester) async {
    var passed = false;
    var index = 0;

    await tester.pumpFrames(
      QueryCacheProvider(
        child: HookBuilder(
          builder: (context) {
            final result = useInfiniteQuery<List<String>>(
              'data',
              (page) {
                final value =
                    page > 2 ? List<String>.empty() : [page.toString()];
                return Future<List<String>>.value(value);
              },
              getFetchMore: (latestPage, allPages) {
                if (latestPage is List && latestPage.isEmpty) {
                  return false;
                } else {
                  return index++;
                }
              },
            );

            if (result.isSuccess && result.canFetchMore) {
              result.fetchMore();
            }

            if (!result.canFetchMore && result.isSuccess && !passed) {
              index = 0;
              passed = true;
              Provider.of<QueryCache>(context).invalidateQueries('data');
            }
            return Container();
          },
        ),
      ),
      Duration(seconds: 5),
    );

    expect(passed, true);
  });
}
